# example 2, bivariate Dirichlet data

# definitions
n <- 1E5 # number of original samples
alpha <- c(1.25, 2, 0.75) # Dirichlet parameters
x2cl <- c(0.3, 0.7) # conditioning planes in x3-direction
nb <- 1E5 # number of bootstrap samples
np <- 200 # number of plot points
color <- colorRampPalette(c("#FFFFFF","#E6E680","#E6BF1A",
                            "#E68000","#FF4026","#993380",
                            "#4D26BF","#262680","#000000")) # color ramp for plots

# samples
rdirichlet <- function(n, alpha) {
   d <- length(alpha)
   x <- sapply(alpha, function(a) {rgamma(n, a, 1)})
   x <- x / matrix(rep(rowSums(x), times = d), ncol = d)
   x <- x[, 1:(d-1), drop = FALSE]; return(x)
}
xs <- rdirichlet(n, alpha)
plot(xs, type = "p", pch = ".", main = "samples")
for (x2c in x2cl) {
   lines(c(0,1-x2c), x2c*c(1,1), col = "white")
   lines(c(1-x2c,1), x2c*c(1,1))
}

# Dirichlet density
ddirichlet <- function(alpha, x) {
   p <- rep(0, nrow(x))
   i <- seq(1,nrow(x))[(rowSums(x) <= 1) & apply(x >= 0, 1, all)] # pnts inside simplex
   B <- prod(gamma(alpha))/gamma(sum(alpha))
   p[i] <- 1/B * apply(cbind(x[i,,drop = FALSE], 1-rowSums(x[i,,drop = FALSE])) ^
                       (rep(1,length(i))%*%t(alpha-1)), 1, prod)
   return(p)
}
x <- (0:np)/np; xg <- rbind(rep(x,np+1), rep(x,each = np+1))
px <- matrix(ddirichlet(alpha, t(xg)), nrow = np+1)
filled.contour(x, x, px, zlim = c(0,10), color = color)
title(main = "Dirichlet density", xlab = "x1", ylab = "x2")

# construct det
d <- det.construct(t(xs), lb = 0, ub = 0) # det for bootstrap
pxd = matrix(det.query(d, xg), nrow = np+1) # p(x1,x2)
filled.contour(x, x, pxd, zlim = c(0,10), color = color)
title(main = "det", xlab = "x1", ylab = "x2")

# analytical marginal and conditional densities
px <- function(x1,x2) {ddirichlet(alpha, matrix(c(x1,x2), nrow = 1))} # p(x1,x2)
px2 <- function(x2) {(x2^(alpha[2]-1) * (1-x2)^(alpha[1]+alpha[3]-1)) /
                     beta(alpha[2],alpha[1]+alpha[3])} # p(x2)
px1cx2 <- function(x1,x2) {px(x1,x2)/px2(x2)} # p(x1|x2)

# loop over conditioning planes
for (x2c in x2cl) {
   # analytical reference
   plot(x, sapply(x, function(x) {px1cx2(x, x2c)}), type = "l", lty = "solid",
        main = paste("p(x1|x2=", toString(x2c), ")", sep = ""), xlab = "x1", ylab = "")

   # p(x1|x2) from det estimator
   xg <- rbind(x, x2c*rep(1,np+1)) # grid
   pxd <- det.query(d,xg) # p(x1,x2)
   px2d <- det.query(det.construct(t(xs[,2,drop = FALSE])), matrix(x2c)) # p(x2)
   lines(x, pxd/px2d, col = "blue", lty = "dashed")

   # bootstrap conditional on x2 = x2c
   x1r <- det.rnd(nb, d, x2c, 2)
   # p(x1|x2) from bootstrap data
   px1cx2d <- det.query(det.construct(x1r[1,,drop = FALSE]), xg[1,,drop = FALSE])
   lines(x, px1cx2d, col = "red", lty = "dotdash")
   if (x2c == x2cl[-1]) {
      legend("right", c("reference","DET","bootstrap"), col = c("black","blue","red"),
             lwd = c(1,1,1), lty = c("solid","dashed","dotdash"))
   }
}
